"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Logger = void 0;
const winston_1 = require("winston");
const DailyRotateFile = require("winston-daily-rotate-file");
const common_1 = require("@nestjs/common");
const lodash_1 = require("lodash");
const http_1 = require("./http");
const levelPriorities = {
    error: 0,
    warn: 1,
    log: 2,
    debug: 3,
    verbose: 4,
};
function getWinstonLevel(level) {
    return level === 'log' ? 'info' : level;
}
function getNestLevel(level) {
    return level === 'info' ? 'log' : level;
}
function filterEmpty(obj) {
    return Object.keys(obj).reduce((map, k) => {
        const v = obj[k];
        if (typeof v === 'number' || !(0, lodash_1.isEmpty)(v)) {
            map[k] = v;
        }
        return map;
    }, {});
}
function getFormat(fn) {
    return winston_1.format.combine(winston_1.format.timestamp({ format: () => +new Date() }), winston_1.format.printf((obj) => {
        obj.level = getNestLevel(obj.level);
        return JSON.stringify(fn ? fn(obj) : obj);
    }));
}
class Logger {
    constructor(options, context) {
        this.options = options;
        this.context = context;
        this.level = 'verbose';
        this.setLevel(options.level);
        this.loggers = {
            winston: (0, winston_1.createLogger)({
                transports: [
                    options.filename && options.dirname
                        ? new DailyRotateFile(Object.assign(Object.assign({ filename: options.filename, dirname: options.dirname, format: getFormat() }, options), { level: 'silly' }))
                        : [],
                    options.http
                        ? new http_1.HttpTransport(Object.assign(Object.assign({ format: getFormat(options.http.payload) }, options.http), { level: 'silly' }))
                        : [],
                ].flat(),
            }),
            nest: new common_1.ConsoleLogger(),
        };
    }
    getContext(context, request) {
        const userContext = ((typeof this.options.context === 'function'
            ? this.options.context(request)
            : { context: this.options.context }) || {});
        if (typeof context === 'object') {
            Object.assign(userContext, context);
        }
        else if (context) {
            userContext.context = context;
        }
        return filterEmpty(userContext);
    }
    all(level, message, context, trace) {
        context = Object.assign(Object.assign({}, this.context), (typeof context === 'object'
            ? context
            : { context: context }));
        if (levelPriorities[level] > levelPriorities[this.level])
            return;
        this.loggers.nest[level](message.message || message, context.context);
        if (!this.loggers.winston.transports.length)
            return;
        const winstonLevel = getWinstonLevel(level);
        if (message instanceof Error || message.message) {
            return this.loggers.winston.log(Object.assign({ level: winstonLevel, message: message.message, trace: [trace || message.stack] }, context));
        }
        return this.loggers.winston.log(Object.assign({ level: winstonLevel, message }, context));
    }
    setLevel(level) {
        if (!level || levelPriorities[level] === undefined)
            return;
        this.level = level;
    }
    error(message, trace, context) {
        this.all('error', message, context, trace);
    }
    log(message, context) {
        this.all('log', message, context);
    }
    warn(message, context) {
        this.all('warn', message, context);
    }
    debug(message, context) {
        this.all('debug', message, context);
    }
    verbose(message, context) {
        this.all('verbose', message, context);
    }
}
exports.Logger = Logger;
